package com.gic.auth.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.ResourceDTO;
import com.gic.auth.entity.TabSysResource;
import com.gic.auth.service.ResourceApiService;
import com.gic.auth.service.ResourceService;
import com.gic.auth.service.UserResourceService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019/8/28 18:28
 */
@Service("resourceApiService")
public class ResourceApiServiceImpl implements ResourceApiService {

    private Logger logger = LogManager.getLogger(ResourceApiServiceImpl.class);

    @Autowired
    private ResourceService resourceService;
    @Autowired
    private UserResourceService userResourceService;

    @Override
    public ServiceResponse<Integer> saveOrUpdateResource(ResourceDTO resourceDTO) {
        boolean repeat = resourceService.isRepeatByResourceName(resourceDTO.getEnterpriseId(), resourceDTO.getResourceName(), resourceDTO.getResourceId());
        if (repeat) {
            logger.info("enterpriseId:{} , resourceName:{}, 资源组名称重复 ", resourceDTO.getEnterpriseId(), resourceDTO.getResourceName());
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源组名称重复");
        }
        if (resourceDTO.getResourceId() == null) {
            // save
            resourceService.save(resourceDTO);
        } else {
            // update
            resourceService.update(resourceDTO);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> delResource(Integer resourceId) {
        TabSysResource resource = resourceService.getResource(resourceId);
        boolean exist = this.userResourceService.existUserResource(resource.getEnterpriseId(), resource.getResourceId());
        if (exist) {
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "该资源组正在使用中，不能删除");
        }
        Integer line = resourceService.delResource(resourceId);
        return ServiceResponse.success(line);
    }

    @Override
    public ServiceResponse<Page<ResourceDTO>> listResource(ResourceDTO resourceDTO, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page<TabSysResource> page = resourceService.listResource(resourceDTO, pageNum, pageSize);
        Page<ResourceDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, ResourceDTO.class);
        List<ResourceDTO> resourceList = resultPage.getResult();
        if (CollectionUtils.isNotEmpty(resourceList)) {
            List<Integer> resourceIds = resourceList.stream().map(ResourceDTO::getResourceId).collect(Collectors.toList());
            Map<Integer, Integer> map = userResourceService.countByResourceIds(resourceDTO.getEnterpriseId(), resourceIds);
            for (ResourceDTO resource : resourceList) {
                Integer count = map.get(resource.getResourceId());
                resource.setUserResourceCount(count == null ? 0 : count);
            }
        }
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<ResourceDTO> getResource(Integer resourceId) {
        TabSysResource resource = resourceService.getResource(resourceId);
        ResourceDTO resourceDTO = EntityUtil.changeEntityByOrika(ResourceDTO.class, resource);
        return ServiceResponse.success(resourceDTO);
    }
}
