package com.gic.auth.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.AuthorizationStatusEnum;
import com.gic.auth.constant.ResourceTypeEnum;
import com.gic.auth.dto.UnionEnterpriseDTO;
import com.gic.auth.entity.TabSysUnionEnterprise;
import com.gic.auth.entity.TabSysUnionEnterpriseResource;
import com.gic.auth.service.UnionEnterpriseApiService;
import com.gic.auth.service.UnionEnterpriseResourceService;
import com.gic.auth.service.UnionEnterpriseService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constants.Constants;
import com.github.pagehelper.PageHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;

@Service("unionEnterpriseApiService")
public class UnionEnterpriseApiServiceImpl implements UnionEnterpriseApiService {
    @Autowired
    private UnionEnterpriseService unionEnterpriseService;
    @Autowired
    private UnionEnterpriseResourceService unionEnterpriseResourceService;

    @Override
    @Transactional
    public ServiceResponse<Integer> saveUnionEnterprise(UnionEnterpriseDTO enterpriseDTO) {
        TabSysUnionEnterprise unionEnterprise = EntityUtil.changeEntityByJSON(TabSysUnionEnterprise.class, enterpriseDTO);
        int i = 0;
        if(enterpriseDTO.getUnionId() == null) {
            unionEnterprise.setCreateTime(new Date());
            unionEnterprise.setStatus(Constants.NORMAL_STATUS);
            unionEnterprise.setAuthorizationStatus(AuthorizationStatusEnum.NORMAL.getCode());
            unionEnterprise.setUpdateTime(new Date());
            i = this.unionEnterpriseService.saveUnionEnterprise(unionEnterprise);
        }else {
            i = this.unionEnterpriseService.updateUnionEnterprise(unionEnterprise);
        }
        if (i > 0) {
            this.saveResource(unionEnterprise.getUnionId(), enterpriseDTO);
        }
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<UnionEnterpriseDTO> getUnionEnterpriseByUnionEnterpriseId(Integer unionEnterrpiseId) {
        TabSysUnionEnterprise enterprise = this.unionEnterpriseService.getUnionEnterpriseByUnionEnterpriseId(unionEnterrpiseId);
        UnionEnterpriseDTO enterpriseDTO = EntityUtil.changeEntityByJSON(UnionEnterpriseDTO.class, enterprise);
        this.getResource(enterprise.getUnionId(), enterpriseDTO);
        return ServiceResponse.success(enterpriseDTO);
    }

    @Override
    @Transactional
    public ServiceResponse<Integer> delUnionEnterprise(Integer unionId, Integer enterpriseId) {
        int i = this.unionEnterpriseService.delUnionEnterprise(unionId);
        if(i > 0){
            int re = this.unionEnterpriseResourceService.delResource(unionId, enterpriseId);
            if(re == 0){
                throw new RuntimeException();
            }
        }
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<UnionEnterpriseDTO> getUnionEnterpriseById(Integer unionId) {
        TabSysUnionEnterprise enterprise = this.unionEnterpriseService.getUnionEnterpriseById(unionId);
        UnionEnterpriseDTO enterpriseDTO = EntityUtil.changeEntityByJSON(UnionEnterpriseDTO.class, enterprise);
        this.getResource(unionId, enterpriseDTO);
        return ServiceResponse.success(enterpriseDTO);
    }

    @Override
    public ServiceResponse<Page<UnionEnterpriseDTO>> pageUnionEnterprise(String search, Integer enterpriseId, Integer pageNum, Integer pageSize, Integer authorizationStatus) {
        PageHelper.startPage(pageNum, pageSize);
        com.github.pagehelper.Page<TabSysUnionEnterprise> page = this.unionEnterpriseService.pageUnionEnterprise(enterpriseId, authorizationStatus, search);
        return ServiceResponse.success(PageHelperUtils.changePageHelperToCurrentPage(page, UnionEnterpriseDTO.class));
    }

    private int saveResource(Integer unionId, UnionEnterpriseDTO enterpriseDTO){
        int i = 0;
        i = this.unionEnterpriseResourceService.saveResource(unionId, enterpriseDTO.getMemberCardResourceId(),
                ResourceTypeEnum.MEMBER_CARD_RESOURCE.getCode(), enterpriseDTO.getEnterpriseId());
        if( i == 0){
            throw new RuntimeException();
        }
        i = this.unionEnterpriseResourceService.saveResource(unionId, enterpriseDTO.getFwhResourceId(),
                ResourceTypeEnum.FWH_RESOURCE.getCode(), enterpriseDTO.getEnterpriseId());
        if( i == 0){
            throw new RuntimeException();
        }
        i = this.unionEnterpriseResourceService.saveResource(unionId, enterpriseDTO.getAppletResourceId(),
                ResourceTypeEnum.APPLET_RESOURCE.getCode(), enterpriseDTO.getEnterpriseId());
        if( i == 0){
            throw new RuntimeException();
        }
        i = this.unionEnterpriseResourceService.saveResource(unionId, enterpriseDTO.getStoreResourceId(),
                ResourceTypeEnum.STORE_RESOURCE.getCode(), enterpriseDTO.getEnterpriseId());
        if( i == 0){
            throw new RuntimeException();
        }
        i = this.unionEnterpriseResourceService.saveResource(unionId, enterpriseDTO.getGoodsResourceId(),
                ResourceTypeEnum.GOODS_RESOURCE.getCode(), enterpriseDTO.getEnterpriseId());
        if( i == 0){
            throw new RuntimeException();
        }
        i = this.unionEnterpriseResourceService.saveResource(unionId, enterpriseDTO.getOrderResourceId(),
                ResourceTypeEnum.ORDER_RESOURCE.getCode(), enterpriseDTO.getEnterpriseId());
        if( i == 0){
            throw new RuntimeException();
        }
        return i;
    }

    private void getResource(Integer unionId, UnionEnterpriseDTO enterpriseDTO){
        TabSysUnionEnterpriseResource resource = this.unionEnterpriseResourceService.getByUnionIdAndType(unionId, ResourceTypeEnum.MEMBER_CARD_RESOURCE.getCode());
        if(resource != null){
            enterpriseDTO.setMemberCardResourceId(resource.getResource());
        }
        resource = this.unionEnterpriseResourceService.getByUnionIdAndType(unionId,ResourceTypeEnum.FWH_RESOURCE.getCode());
        if(resource != null){
            enterpriseDTO.setFwhResourceId(resource.getResource());
        }
        resource = this.unionEnterpriseResourceService.getByUnionIdAndType(unionId, ResourceTypeEnum.APPLET_RESOURCE.getCode());
        if(resource != null){
            enterpriseDTO.setAppletResourceId(resource.getResource());
        }
        resource = this.unionEnterpriseResourceService.getByUnionIdAndType(unionId, ResourceTypeEnum.STORE_RESOURCE.getCode());
        if(resource != null){
            enterpriseDTO.setStoreResourceId(resource.getResource());
        }
        resource = this.unionEnterpriseResourceService.getByUnionIdAndType(unionId, ResourceTypeEnum.GOODS_RESOURCE.getCode());
        if(resource != null){
            enterpriseDTO.setGoodsResourceId(resource.getResource());
        }
        resource = this.unionEnterpriseResourceService.getByUnionIdAndType(unionId, ResourceTypeEnum.ORDER_RESOURCE.getCode());
        if(resource != null){
            enterpriseDTO.setOrderResourceId(resource.getResource());
        }
    }
}
