package com.gic.auth.web.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.dto.RoleDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.qo.RoleListQO;
import com.gic.auth.service.MenuApiService;
import com.gic.auth.service.RoleApiService;
import com.gic.auth.service.UserApiService;
import com.gic.auth.web.vo.RoleDetailVO;
import com.gic.auth.web.vo.RoleSelectVO;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;

@RestController
@RequestMapping("/role")
public class RoleController {

    @Autowired
    private RoleApiService roleApiService;
    @Autowired
    private MenuApiService menuApiService;
    @Autowired
    private UserApiService userApiService;

    @RequestMapping("/get-detail")
    public RestResponse getDetail(Integer id) {
        return ResultControllerUtils.commonResultOne(roleApiService.getById(id), RoleDetailVO.class);
    }

    @RequestMapping("/save")
    public RestResponse save(RoleDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(roleApiService.save(dto));
    }

    @RequestMapping("/update")
    public RestResponse update(RoleDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(roleApiService.update(dto));
    }

    @RequestMapping("/delete")
    public RestResponse delete(Integer id) {
        return ResultControllerUtils.commonResult(roleApiService.delete(id));
    }

    @RequestMapping("/list-role")
    public RestResponse listRole(RoleListQO params) {
        params.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(roleApiService.pageRole(params));
    }

    @RequestMapping("/list-role-no-page")
    public RestResponse listRoleNoPage() {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        return ResultControllerUtils.commonResult(roleApiService.listRole(enterpriseId), RoleSelectVO.class);
    }

    @RequestMapping("/list-menu")
    public RestResponse listMenu() {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<UserDTO> userResponse = userApiService.getUserByEnterpriseId(enterpriseId);
        if (userResponse.isSuccess()) {
            UserDTO admin = userResponse.getResult();
            Integer userId = admin.getUserId();
            ServiceResponse<List<MenuDTO>> menuResponse = menuApiService.getSuperAdminMenu(userId, enterpriseId);
            if (menuResponse.isSuccess()) {
                return RestResponse.success(menuResponse.getResult());
            } else {
                return EnterpriseRestResponse.failure(menuResponse);
            }
        } else {
            return EnterpriseRestResponse.failure(userResponse);
        }
    }
}
