package com.gic.auth.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.AuthCodeDTO;
import com.gic.auth.dto.UnionEnterpriseDTO;
import com.gic.auth.service.AuthCodeApiService;
import com.gic.auth.service.UnionEnterpriseApiService;
import com.gic.auth.web.qo.PageQO;
import com.gic.auth.web.qo.UnionEnterpriseQO;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.UserDetailUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;

@RestController
public class UnionEnterpriseController {
    @Autowired
    private AuthCodeApiService authCodeApiService;
    @Autowired
    private UnionEnterpriseApiService unionEnterpriseApiService;

    @RequestMapping("save-union-enterprise")
    public RestResponse saveUnionEnterprise(UnionEnterpriseQO unionEnterpriseQO){
        RestResponse validateResult = this.validateAuthCode(unionEnterpriseQO);
        if(validateResult != null){
            return validateResult;
        }
        this.authCodeApiService.expireAuthCode(unionEnterpriseQO.getAuthCodeId());
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        UnionEnterpriseDTO enterpriseDTO = EntityUtil.changeEntityByJSON(UnionEnterpriseDTO.class, unionEnterpriseQO);
        enterpriseDTO.setEnterpriseId(enterpriseId);
        ServiceResponse<Integer> response = this.unionEnterpriseApiService.saveUnionEnterprise(enterpriseDTO);
        if(response.isSuccess()){
            return RestResponse.success(response.getResult());
        }
        return EnterpriseRestResponse.failure(response);
    }

    @RequestMapping("union-enterprise-detail")
    public RestResponse getUnionEnterpriseDetail(Integer unionId){
        if(unionId == null){
            return EnterpriseRestResponse.failure(ErrorCode.MISS_PARAMETER);
        }
        ServiceResponse<UnionEnterpriseDTO> response = this.unionEnterpriseApiService.getUnionEnterpriseById(unionId);
        if(response.isSuccess()){
            return RestResponse.success(response.getResult());
        }
        return EnterpriseRestResponse.failure(response);
    }

    @RequestMapping("del-union-enterprise")
    public RestResponse delUnionEnterprise(Integer unionId){
        if(unionId == null){
            return EnterpriseRestResponse.failure(ErrorCode.MISS_PARAMETER);
        }
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<Integer> response = this.unionEnterpriseApiService.delUnionEnterprise(unionId, enterpriseId);
        if(response.isSuccess()){
            return RestResponse.success(response.getResult());
        }
        return EnterpriseRestResponse.failure(response);
    }

    @RequestMapping("page-union-enterprise")
    public RestResponse pageUnionEnterprise(String search, Integer authorizationStatus, PageQO pageQO){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<Page<UnionEnterpriseDTO>> pageServiceResponse = this.unionEnterpriseApiService.
                pageUnionEnterprise(search, enterpriseId, pageQO.getCurrentPage(), pageQO.getPageSize(), authorizationStatus);
        return RestResponse.success(pageServiceResponse.getResult());
    }

    private RestResponse validateAuthCode(UnionEnterpriseQO unionEnterpriseQO){
        ServiceResponse<AuthCodeDTO> authCode = this.authCodeApiService.getAuthCode(unionEnterpriseQO.getAuthCodeId());
        if(authCode.isSuccess()){
            AuthCodeDTO authCodeDTO = authCode.getResult();
            if(authCodeDTO != null){
                if(authCodeDTO.getStatus() == 0){
                    return EnterpriseRestResponse.failure(ErrorCode.AUTHCODE_USED);
                }
                if(authCodeDTO.getExpirationTime().before(new Date())){
                    return EnterpriseRestResponse.failure(ErrorCode.AUTHCODE_EXPIRE);
                }
                if(!authCodeDTO.getAuthCode().equals(unionEnterpriseQO.getAuthCode())){
                    return EnterpriseRestResponse.failure(ErrorCode.AUTHCODE_MISTAKE);
                }
            }else {
                return EnterpriseRestResponse.failure(ErrorCode.AUTHCODE_ERR);
            }
        }else {
            EnterpriseRestResponse.failure(ErrorCode.AUTHCODE_ERR);
        }
        return null;
    }
}
